#include "Context.h"

GameContext::GameContext()
        : pacman(&PacmanTexture::instance()),
          pacman2(&PacmanTexture2::instance()),
          static_objects(),
          dynamic_objects()
{}

GameContext GameContext::clone() {
    GameContext context;
    context.pacman = pacman;
    context.pacman2 = pacman2;
    context.state = state;

    for (auto& obj: static_objects) {
        context.static_objects.emplace_back(obj->clone());
    }
    for (auto& obj : dynamic_objects) {
        context.dynamic_objects.emplace_back(obj->clone());
    }
    return context;
}

void ContextManager::restore_previous_context() {
    if (m_history.size() == 1) {
        m_initial_context = m_history.top().clone();
        return;
    }
    m_initial_context = std::move(m_history.top());
    m_history.pop();
}

void ContextManager::reset(GameContext &&context) {
    while (!m_history.empty()) m_history.pop();
    m_initial_context = context.clone();
    m_history.emplace(m_initial_context.clone());
}
