#include "Menu.h"
#include "../../../../../config.h"
#include "../../../BobBuilder/GameBuilders.h"
#include "../../../State/ThemeManager.h"
#include "../Font/Font.h"
#include <memory>

Menu::Menu(IStateManager& state_manager) : m_state_manager(state_manager) {
    m_directors.push_back(std::make_unique<GameBuilderDirector>(
            std::make_unique<SimpleBuilder>(
                    config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height,
                    config::ROOM_SIZE),
            config::GAME_VIDEO_MODE,
            config::EASY_GAME_TITLE,
            config::EASY_GAME_ENEMY_RATIO));
    m_directors.push_back(std::make_unique<GameBuilderDirector>(
            std::make_unique<SimpleBuilder>(
                    config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height,
                    config::ROOM_SIZE),
            config::GAME_VIDEO_MODE,
            config::MEDIUM_GAME_TITLE,
            config::MEDIUM_GAME_ENEMY_RATIO));
    m_directors.push_back(std::make_unique<GameBuilderDirector>(
            std::make_unique<ComplexBuilder>(
                    config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height,
                    config::ROOM_SIZE),
            config::GAME_VIDEO_MODE,
            config::HARD_GAME_TITLE,
            config::HARD_GAME_ENEMY_RATIO));

    m_choose_button.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 130.f), config::EASY_GAME_TITLE, 24));
    m_choose_button.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 170.f), config::MEDIUM_GAME_TITLE, 24));
    m_choose_button.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 210.f), config::HARD_GAME_TITLE, 24));

    m_choose_button_players.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 255.f), config::ONE_PLAYER_GAME_TITLE, 24));
    m_choose_button_players.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 295.f), config::TWO_PLAYER_GAME_TITLE, 24));

    m_choose_button_theme.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 343.f), config::BUTTON_THEME_WHITE, 24));
    m_choose_button_theme.push_back(std::make_unique<ChooseButton>(sf::Vector2f(400.f, 383.f), config::BUTTON_THEME_BLACK, 24));
    m_choose_button_theme[m_selected_theme]->select();
    m_exit_button = std::make_unique<Button>(sf::Vector2f(900.f, 500.f), sf::Vector2f(150.f, 50.f), config::BUTTON_TEXT_EXIT, 24, std::make_unique<ExitCommand>(state_manager));
    m_proceed_button = std::make_unique<Button>(sf::Vector2f(450.f, 500.f), sf::Vector2f(150.f, 50.f), config::BUTTON_START_GAME, 24,nullptr);
}

void Menu::process_mouse(sf::Vector2f pos, bool is_pressed) {
    if (is_pressed) {
        ///> выбор уровня
        for (size_t i = 0; i < m_choose_button.size(); i++) {
            if (m_choose_button[i]->contains(pos)) {
                for (auto& button : m_choose_button)
                    button->unselect();
                m_choose_button[i]->select();
                m_selected_level = i;
            }
        }
        ///> exit
        if (m_exit_button->is_position_in(pos))
            m_exit_button->push();
        ///> выбор количества игроков
        for (int i = 0; i < m_choose_button_players.size(); i++) {
            if (m_choose_button_players[i]->contains(pos)) {
                for (auto& button : m_choose_button_players)
                    button->unselect();
                m_choose_button_players[i]->select();
                m_selected_players = i;
            }
        }
        ///> выбор темы приложения
        for (int i = 0; i < m_choose_button_theme.size(); i++) {
            if (m_choose_button_theme[i]->contains(pos)) {
                for (auto& button : m_choose_button_theme)
                    button->unselect();
                m_choose_button_theme[i]->select();
                m_selected_theme = i;
                if  (i == 0)
                    ThemeManager::Instance().set_theme(ThemeManager::Theme::LIGHT);
                else
                    ThemeManager::Instance().set_theme(ThemeManager::Theme::DARK);
                break;
            }
        }
        ///> создание команды при нажатии на go to game
        if (m_selected_level >= 0 && m_proceed_button->is_position_in(pos)) {
            auto command = std::make_unique<GameCommand>(m_state_manager, m_directors[m_selected_level], m_selected_players +1);
            command->execute();
        }
    }
}

void Menu::draw_into(sf::RenderWindow &window) {
    sf::Text title(config::SELECT_LEVEL_TITLE_WINDOW, MyFont::Instance(), 40);
    title.setPosition(window.getSize().x/2.f-100.f, 50.f);
    title.setFillColor(ThemeManager::Instance().getTitleTextColor());
    window.draw(title);
    sf::Text title2(config::SELECT_LEVEL_TITLE, MyFont::Instance(), 30);
    title2.setPosition(window.getSize().x/2.f-350.f, 120.f);
    title2.setFillColor(ThemeManager::Instance().getTitleTextColor());
    window.draw(title2);
    for (auto& button : m_choose_button)
        button->draw_into(window);
    sf::Text title3(config::SELECT_LEVEL_TITLE_PLAYER, MyFont::Instance(), 30);
    title3.setPosition(window.getSize().x/2.f-350.f, 240.f);
    title3.setFillColor(ThemeManager::Instance().getTitleTextColor());
    window.draw(title3);
    for (auto& button : m_choose_button_players)
        button->draw_into(window);
    sf::Text title4(config::SELECT_THEME, MyFont::Instance(), 30);
    title4.setPosition(window.getSize().x/2.f-350.f, 330.f);
    title4.setFillColor(ThemeManager::Instance().getTitleTextColor());
    window.draw(title4);
    for (auto& button : m_choose_button_theme)
        button->draw_into(window);
    m_proceed_button->draw_into(window);
    m_exit_button->draw_into(window);
}

//Menu::Menu(IStateManager& state_manager) {
//    float delta = 20;
//    sf::Vector2f starting_position(80,5);
//    auto easy_director = std::make_unique<GameBuilderDirector>(
//            std::make_unique<SimpleBuilder>(
//                    config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height,
//                    config::ROOM_SIZE),
//            config::GAME_VIDEO_MODE,
//            config::EASY_GAME_TITLE,
//            config::EASY_GAME_ENEMY_RATIO);
//    auto medium_director = std::make_unique<GameBuilderDirector>(
//            std::make_unique<SimpleBuilder>(
//                    config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height,
//                    config::ROOM_SIZE),
//            config::GAME_VIDEO_MODE,
//            config::MEDIUM_GAME_TITLE,
//            config::MEDIUM_GAME_ENEMY_RATIO);
//    auto hard_director = std::make_unique<GameBuilderDirector>(
//            std::make_unique<ComplexBuilder>(
//                    config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height,
//                    config::ROOM_SIZE),
//            config::GAME_VIDEO_MODE,
//            config::HARD_GAME_TITLE,
//            config::HARD_GAME_ENEMY_RATIO);
//
//    auto easy_command = std::make_unique<GameCommand>(state_manager, std::move(easy_director));
//    auto medium_command = std::make_unique<GameCommand>(state_manager, std::move(medium_director));
//    auto hard_command = std::make_unique<GameCommand>(state_manager, std::move(hard_director));
//    auto exit_command = std::make_unique<ExitCommand>(state_manager);
//
//    m_buttons.emplace_back(std::make_unique<Button>(sf::Vector2f{starting_position.x, starting_position.y + 1 * config::BUTTON_SIZE.y + 0 * delta}, config::BUTTON_SIZE, config::BUTTON_TEXT_EASY, config::BUTTON_FONT_SIZE , std::move(easy_command)));
//    m_buttons.emplace_back(std::make_unique<Button>(sf::Vector2f{starting_position.x, starting_position.y + 2 * config::BUTTON_SIZE.y + 1 * delta}, config::BUTTON_SIZE, config::BUTTON_TEXT_MEDIUM, config::BUTTON_FONT_SIZE , std::move(medium_command)));
//    m_buttons.emplace_back(std::make_unique<Button>(sf::Vector2f{starting_position.x, starting_position.y + 3 * config::BUTTON_SIZE.y + 2 * delta}, config::BUTTON_SIZE, config::BUTTON_TEXT_HARD, config::BUTTON_FONT_SIZE , std::move(hard_command)));
//    m_buttons.emplace_back(std::make_unique<Button>(sf::Vector2f{starting_position.x, starting_position.y + 4 * config::BUTTON_SIZE.y + 3 * delta}, config::BUTTON_SIZE, config::BUTTON_TEXT_EXIT, config::BUTTON_FONT_SIZE, std::move(exit_command)));
//}

//void Menu::draw_into(sf::RenderWindow &window) {
//    //@todo change to const Button&
//    for (auto& ptr_button : m_buttons)
//        ptr_button->draw_into(window);
//}

//void Menu::process_mouse(sf::Vector2f pos, bool is_pressed) {
//    for (auto& ptr_button : m_buttons) {
//        if (ptr_button->is_position_in(pos)) {
//            ptr_button->select();
//            if (is_pressed) ptr_button->push();
//        }
//        else ptr_button->unselect();
//    }
//}