#include "IGameEvent.h"
#include "../Context/Context.h"

void LostGame::handle(GameContext* context) {
    context->state = GameContext::LOST;
}

void DeleteStaticEntity::handle(GameContext* context) {
    auto it = std::find_if(context->static_objects.begin(), context->static_objects.end(),
                           [this](const std::unique_ptr<IStaticEntity>& ptr) { return ptr.get() == m_ptr_entity; });
    context->static_objects.erase(it);
    notify();
}

DeleteStaticEntity::DeleteStaticEntity(IStaticEntity* ptr_entity) :
        m_ptr_entity(ptr_entity) {
}

void WinGame::handle(GameContext* context) {
    context->state = GameContext::WIN;
}

void IGameEvent::add_observer(IObserver *observer) {
    m_observers.push_back(observer);
}

void IGameEvent::notify() const {
    for (auto observer : m_observers)
        observer->on_notify(*this);
}
