#pragma once
#include "../State/IState.h"
#include "../BobBuilder/BobBuilderDirector.h"

#include <memory>

class ISelectCommand {
public:
    virtual ~ISelectCommand() = default;
    virtual void execute() = 0;
};

class ChangeStateCommand : public ISelectCommand {
public:
    explicit ChangeStateCommand(IStateManager& state_manager) : m_state_manager(state_manager) {}
protected:
    IStateManager& m_state_manager;
};

class ExitCommand : public ChangeStateCommand {
public:
    explicit ExitCommand(IStateManager& state_manager) : ChangeStateCommand(state_manager) {}
    void execute() override {
        m_state_manager.set_next_state(std::make_unique<ExitState>(m_state_manager));
    }
};

class GameCommand: public ChangeStateCommand {
public:
    GameCommand(IStateManager& state_manager, std::unique_ptr<GameBuilderDirector>& ptr_director, int players) :
            ChangeStateCommand(state_manager), m_ptr_director(std::move(ptr_director)), m_players(players) {};
    void execute() override {
        auto state = m_ptr_director->build(m_state_manager, m_players);
        m_state_manager.set_next_state(std::move(state));
    };

private:
    int m_players;
    std::unique_ptr<GameBuilderDirector> m_ptr_director;
};